#!/usr/bin/python

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.six.moves.urllib.parse import quote_plus

DOCUMENTATION = '''
---
module: ovh_loadbalancer
short_description: Manage OVH LoadBalancer
description:
    - Manage for OVH (French European hosting provider) LoadBalancer.
author: "Emmanuel Cohen (@atmaniak)"
notes:
    - Uses the python OVH Api U(https://github.com/ovh/python-ovh).
      You have to create an application (a key and secret) with a consummer
      key as described into U(https://eu.api.ovh.com/g934.first_step_with_api)
requirements:
    - ovh >=  0.4.8
options:
    loadbalancerid:
        required: true
        description:
            - Id of loadbalancer to manage
    refresh:
        required: true
        choices: ['true', 'false']
        description:
            - Refresh the loadbalancer configuration
    timeout:
        required: false
        default: 120
        description:
            - Time to wait for the refresh to complete
    endpoint:
        required: false
        default: ovh-eu
        description:
            - The endpoint to use (for instance ovh-eu)
    application_key:
        required: true
        description:
            - The applicationKey to use
    application_secret:
        required: true
        description:
            - The application secret to use
    consumer_key:
        required: true
        description:
            - The consumer key to use

'''

EXAMPLES = '''
# Refresh LoadBalancer 1234
- ovh_loadbalancer:
    name: 1234
    refresh: true
    application_key: yourkey
    application_secret: yoursecret
    consumer_key: yourconsumerkey
'''

RETURN = '''
'''
import time  # noqa: E402

try:
    import ovh
    import ovh.exceptions
    from ovh.exceptions import APIError
    HAS_OVH = True
except ImportError:
    HAS_OVH = False


def getOvhClient(ansibleModule):
    endpoint = ansibleModule.params.get('endpoint')
    application_key = ansibleModule.params.get('application_key')
    application_secret = ansibleModule.params.get('application_secret')
    consumer_key = ansibleModule.params.get('consumer_key')

    return ovh.Client(
        endpoint=endpoint,
        application_key=application_key,
        application_secret=application_secret,
        consumer_key=consumer_key
    )


def waitForNoTask(client, loadbalancerid, timeout):
    currentTimeout = timeout
    while client.get('/ipLoadbalancing/{0}/task'.format(quote_plus(loadbalancerid)),
                     action='refreshIplb',
                     status='todo'):
        time.sleep(1)  # Delay for 1 sec
        currentTimeout -= 1
        if currentTimeout < 0:
            return False
    return True


def waitForTaskDone(client, loadbalancerid, taskId, timeout):
    currentTimeout = timeout
    while True:
        task = client.get('/ipLoadbalancing/{0}/task/{1}'.format(quote_plus(loadbalancerid), taskId))
        if task['status'] == 'done':
            return True
        time.sleep(5)  # Delay for 5 sec because it's long to wait completion, do not harass the API
        currentTimeout -= 5
        if currentTimeout < 0:
            return False
    return True


def main():
    module = AnsibleModule(
        argument_spec=dict(
            loadbalancerid=dict(required=True),
            refresh=dict(type='bool', default=True),
            timeout=dict(default=120, type='int'),
            endpoint=dict(default='ovh-eu'),
            wait_completion=dict(default=True, type='bool'),
            wait_task_completion=dict(default=0, type='int'),
            application_key=dict(required=True, no_log=True),
            application_secret=dict(required=True, no_log=True),
            consumer_key=dict(required=True, no_log=True),
        ),
        supports_check_mode=True
    )

    result = dict(
        changed=False
    )

    if not HAS_OVH:
        module.fail_json(msg='ovh-api python module is required to run this module ')

    # Get parameters
    loadbalancerid = module.params.get('loadbalancerid')
    refresh = module.params.get('refresh')
    timeout = module.params.get('timeout')
    wait_completion = module.params.get('wait_completion')
    wait_task_completion = module.params.get('wait_task_completion')

    # Connect to OVH API
    client = getOvhClient(module)

    # Check that the load balancer exists
    try:
        client.get('/ipLoadbalancing/{0}'.format(quote_plus(loadbalancerid)))
    except APIError as apiError:
        module.fail_json(
            msg='Unable to call OVH api for getting the list of ips, '
                'check application key, secret, consumerkey and parameters. '
                'Error returned by OVH api was0 : {0}'.format(apiError))

    # Check that no task is pending before going on
    try:
        if not waitForNoTask(client, loadbalancerid, timeout):
            module.fail_json(
                msg='Timeout of {0} seconds while waiting for no pending '
                    'tasks before executing the module '.format(timeout))
    except APIError as apiError:
        module.fail_json(
            msg='Unable to call OVH api for getting the list of pending tasks '
                'of the ip, check application key, secret, consumerkey '
                'and parameters. Error returned by OVH api was1 : {0}'
                .format(apiError))

    if not module.check_mode and refresh:
        try:
            if wait_task_completion == 0:
                task = client.post('/ipLoadbalancing/{}/refresh'.format(loadbalancerid))
                taskId = task['id']
                result['changed'] = True
            else:
                # Just wait for the given taskId to be completed
                taskId = wait_task_completion
            result['TaskId'] = taskId
            if wait_completion or wait_task_completion != 0:
                if not waitForTaskDone(client, loadbalancerid, taskId, timeout):
                    module.fail_json(
                        msg='Timeout of {0} seconds while waiting for completion '
                            'of move ip to service'.format(timeout))
        except APIError as apiError:
            module.fail_json(
                msg='Unable to call OVH api for getting the object '
                'check application key, secret, consumerkey and parameters. '
                'Error returned by OVH api was2222 : {0}'.format(apiError))

    module.exit_json(**result)


if __name__ == '__main__':
    main()
